@@ -2,13 +2,10 @@ import os
 from shutil import rmtree
 
 from .click import secho
-# Patch by vphilippon 2017-11-22: Use pipenv cache path.
-# from ._compat import user_cache_dir
-from pipenv.environments import PIPENV_CACHE_DIR
+from ._compat import user_cache_dir
 
 # The user_cache_dir helper comes straight from pipenv.patched.notpip itself
-# CACHE_DIR = user_cache_dir(os.path.join('pip-tools'))
-CACHE_DIR = PIPENV_CACHE_DIR
+CACHE_DIR = user_cache_dir('pip-tools')
 
 # NOTE
 # We used to store the cache dir under ~/.pip-tools, which is not the
